package com.lqkj.link.config.LimitRequest;

import com.lqkj.link.message.MessageBean;
import com.lqkj.link.util.RSAUtils;
import jakarta.servlet.http.HttpServletRequest;
import net.jodah.expiringmap.ExpirationPolicy;
import net.jodah.expiringmap.ExpiringMap;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.aspectj.lang.reflect.MethodSignature;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestAttributes;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import java.lang.reflect.Method;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.TimeUnit;

@Aspect
@Component
public class LimitRequestAspect {

    private static final ConcurrentHashMap<String, ExpiringMap<String, Integer>> book = new ConcurrentHashMap<>();

    @Pointcut(value = "@within(com.lqkj.link.config.LimitRequest.LimitRequest)")
    public void excludeService() {

    }

    @Around(value = "excludeService()", argNames = "pjp")
    public Object doAround(ProceedingJoinPoint pjp) throws Throwable {
        RequestAttributes ra = RequestContextHolder.getRequestAttributes();
        ServletRequestAttributes sra = (ServletRequestAttributes) ra;
        assert sra != null;
        HttpServletRequest request = sra.getRequest();

        // 从header获取RSA加密后的AES KEY
        String aesKey = request.getHeader("KEY");
        aesKey = RSAUtils.decryptBase64(aesKey);

        ExpiringMap<String, Integer> uc = book.getOrDefault(request.getRequestURI(), ExpiringMap.builder().variableExpiration().build());

        Integer uCount = uc.getOrDefault(request.getRemoteAddr(), 0);

        // 获取类上的注解
        Class<?> clazz = pjp.getTarget().getClass();
        LimitRequest limitRequest = clazz.getAnnotation(LimitRequest.class);

        // 获取接口上的注解，如果有，以接口上的注解为准
        MethodSignature ms = (MethodSignature) pjp.getSignature();
        Method targetMethod = clazz.getDeclaredMethod(ms.getName(), ms.getParameterTypes());

        LimitRequest funLimitRequest = targetMethod.getAnnotation(LimitRequest.class);
        if (funLimitRequest != null) {
            limitRequest = funLimitRequest;
        }

        if (limitRequest.limit()) {
            if (uCount >= limitRequest.count()) {
                return MessageBean.error("接口请求超过次数");
            } else if (uCount == 0) {
                uc.put(request.getRemoteAddr(), uCount + 1, ExpirationPolicy.CREATED, limitRequest.time(), TimeUnit.MILLISECONDS);
            } else {
                uc.put(request.getRemoteAddr(), uCount + 1);
            }
            book.put(request.getRequestURI(), uc);
        }

        return pjp.proceed();
    }
}
