package com.lqkj.link.module.audit.repository;

import com.lqkj.link.module.audit.domain.AuditRecord;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.query.Param;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;

@Repository
public interface AuditRecordRepository extends JpaRepository<AuditRecord, Integer> {

    @Query(nativeQuery = true,
            value = "select record_id as \"recordId\", aui.user_code as \"auditorId\", " +
                        "aui.display_name as auditor, zi.zone_name as \"zoneName\", " +
                        "aui1.user_code as \"authorId\", aui1.display_name as author, " +
                        "to_char(ar.apply_time, 'yyyy-mm-dd hh24:mi:ss') as \"applyTime\", " +
                        "to_char(ar.auth_time, 'yyyy-mm-dd hh24:mi:ss') as \"auditTime\", " +
                        "ar.auth_status as \"authStatus\" " +
                    "from audit_record ar " +
                    "inner join zone_info zi on ar.zone_id = zi.zone_id " +
                    "inner join user_info aui1 on zi.user_id = aui1.user_id " +
                    "left join user_info aui on ar.user_id = aui.user_id " +
                    "where (:auditor = '' or aui.display_name like concat('%', :auditor, '%')) " +
                    "and (:author = '' or aui1.display_name like concat('%', :author, '%')) " +
                    "and (:authStatus = -1 or ar.auth_status = :authStatus)" +
                    "order by ar.auth_time desc, apply_time desc",
            countQuery = "select count(*) " +
                    "from audit_record ar " +
                    "inner join zone_info zi on ar.zone_id = zi.zone_id " +
                    "inner join user_info aui1 on zi.user_id = aui1.user_id " +
                    "left join user_info aui on ar.user_id = aui.user_id " +
                    "where (:auditor = '' or aui.display_name like concat('%', :auditor, '%')) " +
                    "and (:author = '' or aui1.display_name like concat('%', :author, '%')) " +
                    "and (:authStatus = -1 or (:authStatus = 4 and ar.delete_status is true) or ar.auth_status = :authStatus)"
    )
    Page<Map<String, Object>> pageQuery(@Param("auditor") String auditor,
                                        @Param("author") String author,
                                        @Param("authStatus") Integer authStatus,
                                        Pageable pageable);

    @Modifying
    @Query(nativeQuery = true,
        value = "update audit_record set auth_status = 4 where record_id in :recordIds"
    )
    void batchDelete(@Param("recordIds") List<Integer> recordIds);

    @Query(nativeQuery = true,
        value = "select * from audit_record where zone_id = :zoneId and auth_status = 1"
    )
    AuditRecord findAuditing(@Param("zoneId") Integer zoneId);
}
