package com.lqkj.link.module.authority.repository;

import com.lqkj.link.module.authority.domain.RoleInfo;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.query.Param;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;

@Repository
public interface RoleInfoRepository extends JpaRepository<RoleInfo, Integer> {

    @Query(nativeQuery = true,
        value = "select ri.* from role_info ri, user_role ur where ur.user_id = :userId and ur.role_id = ri.role_id order by ri.role_id"
    )
    List<RoleInfo> listWithUserId(@Param("userId") Integer userId);

    @Query(nativeQuery = true,
        value = "select string_agg(ri.role_name, ',') from user_role ur, role_info ri where ur.role_id = ri.role_id and ur.user_id = :userId"
    )
    String findRoleNames(@Param("userId") Integer userId);

    @Modifying
    @Query(nativeQuery = true,
        value = "with t1 as(delete from user_role where user_id = :userId and role_id not in :roleIds)" +
                "insert into user_role(role_id, user_id) " +
                "select role_id, :userId from role_info where role_id in :roleIds " +
                "on conflict (role_id, user_id) do nothing "
    )
    void updateUserRole(@Param("userId") Integer userId,
                        @Param("roleIds") List<Integer> roleIds);

    @Query(nativeQuery = true,
            value = "with t1 as(select * from role_info where role_name like concat('%', :roleName, '%')), " +
                    "t2 as(select ra.role_id, string_agg(distinct(authority_name), ',') as auths from role_authority ra, authority_info ai " +
                        "where ra.authority_id = ai.authority_id " +
                            "and ra.role_id in (select role_id from t1) " +
                        "group by ra.role_id) " +
                    "select t1.role_id as \"roleId\", t1.role_name as \"roleName\", t2.auths, " +
                        "to_char(t1.update_time, 'yyyy-mm-dd hh24:mi:ss') as \"updateTime\" " +
                    "from t1, t2 where t1.role_id = t2.role_id " +
                    "order by t1.role_id")
    Page<Map<String, Object>> pageQuery(@Param("roleName") String roleName,
                                        Pageable pageable);

    @Query(nativeQuery = true,
            value = "select * from role_info where role_name like concat('%', :roleName, '%') and role_id != 1 order by role_id")
    Page<RoleInfo> pageQueryV2(@Param("roleName") String roleName,
                                        Pageable pageable);

    @Query(nativeQuery = true,
        value = "select * from role_info where role_id != 1 order by role_id"
    )
    List<RoleInfo> queryAll();

    @Query(nativeQuery = true,
        value = "select string_agg(distinct (authority_name), ',') " +
                "from role_authority ra , authority_info ai " +
                "where ra.role_id = :roleId and ra.authority_id = ai.authority_id"
    )
    String queryAuthNamesWithRole(@Param("roleId") Integer roleId);

    @Query(nativeQuery = true,
        value = "select ai.authority_id from authority_info ai, role_authority ra " +
                "where ra.role_id = :roleId and ai.authority_id = ra.authority_id " +
                "order by ai.authority_id"
    )
    List<Map<String, Object>> queryAuthsWithRole(@Param("roleId") Integer roleId);

    @Query(nativeQuery = true,
        value = "select count(*) > 0 from role_info where en_name = :enName"
    )
    Boolean hasSameEnName(@Param("enName") String enName);

    @Query(nativeQuery = true,
            value = "select count(*) > 0 from role_info where role_name = :roleName"
    )
    Boolean hasSameName(@Param("roleName") String roleName);

    @Query(nativeQuery = true,
            value = "select count(*) > 0 from role_info where en_name = :enName and role_id != :roleId"
    )
    Boolean hasSameEnNameWithoutOneId(@Param("enName") String enName,
                                      @Param("roleId") Integer roleId);

    @Query(nativeQuery = true,
            value = "select count(*) > 0 from role_info where role_name = :roleName and role_id != :roleId"
    )
    Boolean hasSameNameWithoutOneId(@Param("roleName") String roleName,
                                    @Param("roleId") Integer roleId);

    @Modifying
    @Query(nativeQuery = true,
        value = "with t1 as (delete from role_authority where role_id = :roleId and authority_id not in :authorityIds)" +
                "insert into role_authority(role_id, authority_id) " +
                "select :roleId, authority_id from authority_info where authority_id in :authorityIds " +
                "on conflict (role_id, authority_id) do nothing "
    )
    void updateRoleAuthority(@Param("roleId") Integer roleId,
                             @Param("authorityIds") List<Integer> authorityIds);

    @Query(nativeQuery = true,
        value = "select ai.authority_id, ai.parent_id, ai.authority_name " +
                "from authority_info ai, role_authority ra , user_role ur , user_info ui " +
                "where ai.authority_id = ra.authority_id " +
                    "and ra.role_id = ur.role_id " +
                    "and ur.user_id = ui.user_id " +
                    "and ui.user_code = :userCode " +
                "order by ai.parent_id, ai.authority_id"
    )
    List<Map<String, Object>> queryAuthsByUser(String userCode);


    @Modifying
    @Query(nativeQuery = true,
        value = "delete from role_info where role_id in :roleIds"
    )
    void deleteNoUserRole(List<Integer> roleIds);

    @Query(nativeQuery = true,
        value = "select distinct (role_id) from user_role where role_id in :roleIds"
    )
    List<Integer> findHasUserRole(@Param("roleIds") List<Integer> roleIds);

    @Query(nativeQuery = true,
        value = "select string_agg(role_name, ',') from role_info where role_id in :hasUserRoleIds"
    )
    String findHasUserRoleNames(@Param("hasUserRoleIds") List<Integer> hasUserRoleIds);
}
