package com.lqkj.link.module.zone.repository;

import com.lqkj.link.module.zone.domain.ModelCategory;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.query.Param;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;

@Repository
public interface ModelCategoryRepository extends JpaRepository<ModelCategory, Integer> {

    @Query(nativeQuery = true,
        value = "select * from model_category where " +
                "user_id is null " +
                "and (:categoryName = '' or category_name like concat('%', :categoryName, '%')) " +
                "order by order_id"
    )
    Page<ModelCategory> pageQuery(@Param("categoryName") String categoryName,
                                  Pageable pageable);

    @Query(nativeQuery = true,
            value = "select category_id as \"categoryId\", icon, category_name as \"categoryName\", mc.user_id is null as official," +
                    "category_id as \"Name\" " +
                    "from model_category mc left join user_info ui on mc.user_id = ui.user_id " +
                    "where mc.user_id is null or ui.user_code = :userCode " +
                    "order by order_id, category_id"
    )
    List<Map<String, Object>> queryWithUserCode(@Param("userCode") String userCode);

    @Query(nativeQuery = true,
        value = "select count(*) > 0 from model_category where category_name = :categoryName"
    )
    Boolean hasSameName(@Param("categoryName") String categoryName);

    @Query(nativeQuery = true,
            value = "select count(*) > 0 from model_category where category_name = :categoryName and category_id != :categoryId"
    )
    Boolean hasSameNameWithoutOne(@Param("categoryName") String categoryName,
                                  @Param("categoryId") Integer categoryId);

    @Query(nativeQuery = true,
            value = "select count(*) > 0 from model_category where category_name = :categoryName and user_id = :userId"
    )
    Boolean hasSameNameByUser(@Param("categoryName") String categoryName,
                              @Param("userId") Integer userId);

    @Query(nativeQuery = true,
            value = "select count(*) > 0 from model_category " +
                    "where category_name = :categoryName and category_id != :categoryId and user_id = :userId"
    )
    Boolean hasSameNameByUserWithoutOne(@Param("categoryName") String categoryName,
                                        @Param("categoryId") Integer categoryId,
                                        @Param("userId") Integer userId);

}
